;--------------------------------------------------------
; IDE ATAPI CD-ROM Driver for IDE64 controller
; Copyright (C) 1998,1999 by Josef Soucek (josef.soucek@spt.cz) 
;
; Many thanks to Alexis Ledoux (COM40011@ITCARLOW.IE)
; Wolfgang Sauer (w_sauer@gmx.net) and
; Jonas Bjurman (jonbj103@student.liu.se)
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; Revision History:
;
; 98/01/10:   Project started
; 99/10/05:   Created these API functions: ioctl_CDROMUNLOCK, ioctl_CDROMEJECT
;             ioctl_CDROMCLOSETRAY, ioctl_CDROMLOCK, ioctl_CDROMSTART
;             ioctl_CDROMSTOP, ioctl_CDROMPLAYTRKIND
; 99/10/25:   ioctl_CDROMRESET
;
;--------------------------------------------------------
; Info:
; Some commands leads in the "ATTENTION MODE". This mode
; causes an error for the following command, even if command
; is set up correctly. After one failed command, the Attention
; mode is cleared. Some examples for this mode is eny kind of 
; reset and closing tray etc.
;--------------------------------------------------------

START =*

;-----
;PORTS
;-----

IDE_DATA_REG      = $de20
IDE_ERROR_REG     = $de21
IDE_NSECTOR_REG   = $de22       ;Interrupt reason register
IDE_SECTOR_REG    = $de23
IDE_LCYL_REG      = $de24
IDE_HCYL_REG      = $de25
IDE_SELECT_REG    = $de26       ;Head
IDE_STATUS_REG    = $de27
IDE_CONTROL_REG   = $de2e
IDE_FEATURE_REG   = IDE_ERROR_REG
IDE_COMMAND_REG   = IDE_STATUS_REG
IDE_ALTSTATUS_REG = IDE_CONTROL_REG

low_reg           = $de30  ;pomocny reg. 0-7
hig_reg           = $de31  ;pomocny reg. 8-15
service_port      = $de40

;SCPU REG

scpu_normal = $d07a
scpu_turbo  = $d07b

;--------------   
;ATAPI COMMANDS
;--------------
ATAPI_CMD_START_STOP  =$1B  ; eject/load
ATAPI_CMD_LOCK_UNLOCK =$1E  ; Lock / Unlock the door
ATAPI_CMD_READ10      =$28  ; read data sector(s)
ATAPI_CMD_READ12      =$A8  ; read data sector(s)
ATAPI_CMD_READTOC     =$43  ; read audio table-of-contents
ATAPI_CMD_PLAY        =$47  ; play audio
ATAPI_CMD_PAUSE       =$4B  ; pause/continue audio
ATAPI_CMD_STOP        =$4E  ; Stop the play
ATAPI_CMD_SPEED       =$BB  ; Speed of read and write
ATAPI_READ_CAPACITY   =$25  ; Read total Capacity of CDROM
ATAPI_READ_SUB        =$42  ; Read sub channel
ATAPI_MODE_SENSE_10   =$5A
ATAPI_MODE_SELECT_10  =$55
ATAPI_READCD          =$BE

atapi_reset_d     = $08
atapi_ident_d     = $a1
atapi_service_d   = $a2
WIN_PACKETCMD     = $a0
INQUIRY           = $12

TEST_UNIT_READY   = $00
REQUEST_SENSE     = $03

;--------------------
;COMMAND COMP. STATUS
;--------------------

ATAPI_PH_ABORT    = 0    ; other possible phases (1, 2,
ATAPI_PH_DONE     = 3    ; and 11) are invalid
ATAPI_PH_DATAOUT  = 8
ATAPI_PH_CMDOUT   = 9
ATAPI_PH_DATAIN   = 10

;------
;KERNAL
;------

chrout       = $ffd2

;---------
;VARIABLES
;---------
;------------------
             *= $00f6

pom1         *=*+1
pom2         *=*+1
pom3         *=*+1
pom4         *=*+1
pom5         *=*+1
pom6         *=*+1
pom7         *=*+1
pom8         *=*+1    ;Pouziva tst_BUSY
drive        *=*+1    ;!!! mesmi se pouzit adresa $ff - vyuziva print routina
;------------------
             *= $1000
             
flag         *=*+1
out_buffer   *=*+1
NumTracks    *=*+1             
Start_M      *=*+1
Start_S      *=*+1
Start_F      *=*+1
End_M        *=*+1
End_S        *=*+1
End_F        *=*+1

             *= $1100

atapi_request_sense        =*        ;max 18 bajtu
reqbuf_error_code         *=*+1
reqbuf_reserved1          *=*+1
reqbuf_sense_key          *=*+1
reqbuf_info               *=*+4
reqbuf_sense_len          *=*+1
reqbuf_command_info       *=*+4
reqbuf_asc                *=*+1
reqbuf_ascq               *=*+1
reqbuf_fru                *=*+1
reqbuf_sense_key_specific *=*+3

             *= $1200                ;Pocatek command bufferu

comm_buff                  =*

             *= $1300
toc_buffer                 =*
identify_buffer            =*

tocbuf_data_hi            *=*+1
tocbuf_data_low           *=*+1
tocbuf_FirstTrk           *=*+1
tocbuf_LastTrk            *=*+1
;
toc_data_buffer            =*

;--------------------------------------
port     .macro ap
         pha
         lda #'ap'
         sta service_port
         pla
         .endm
pc       .macro ad,num
         lda #'num'
         sta comm_buff+'ad'
         .endm
pm       .macro ad,num
         lda 'num'
         sta comm_buff+'ad'
         .endm
pa       .macro ad
         sta comm_buff+'ad'
         .endm
sto      .macro ad,num
         lda #'num'
         sta 'ad'
         .endm
memset   .macro ap
         pha
         lda #'ap'
         ldx #0
memset_loop = *
         sta comm_buff,x
         inx
         cpx #12
         bne memset_loop
         pla
         .endm
return   .macro ap
         lda #'ap'
         rts
         .endm
;--------------------------------------
 
;------------------
;Minix
;
;CDIOPLAYTI
;CDIOPLAYMSS
;CDIOREADTOCHDR
;CDIOREADTOC
;CDIOREADSUBCH
;CDIOREADTIME
;CDIOREADDATA
;CDIOVOLREAD
;CDIOVOLCTRL
;CDIOSTOP
;CDIOPAUSE
;CDIORESUME
;CDIOPLAY
;CDIOREADCAP
;CDIOCHKSTATUS
;CDIOCLOSE
;CDIOLOCK
;CDIOUNLOCK
;------------------
;Linux
;
;CDROMEJECT
;CDROMCLOSETRAY
;CDROMEJECT_SW
;CDROMPAUSE
;CDROMRESUME
;CDROMSTART
;CDROMSTOP
;CDROMPLAYMSF
;CDROMPLAYTRKIND
;CDROMREADTOCHDR
;CDROMREADTOCENTRY
;CDROMSUBCHNL
;CDROMVOLCTRL
;CDROMVOLREAD
;CDROMMULTISESSION
;CDROMREADAUDIO
;CDROMREADMODE1
;CDROMREADMODE2
;CDROM_GET_UPC
;CDROMLOADFROMSLOT
;CDROM_SELECT_DISC
;CDROMRESET
;------------------

;--------------------------------------
;--------------------------------------
;Pozor reset zpusobuje ATTENTION MODE !
;--------------------------------------

                   *= START
;--------------------------------------
; ioctl_SELECTDRIVE
;
; Input: A - drive num. (0=master,1=salve)
;--------------------------------------
ioctl_SELECTDRIVE  = *
                   sta drive
                   rts
;------------------
ioctl_CDROMUNLOCK  = *
                   lda #0
                   sta flag
                   jsr cdrom_lockdoor
                   jsr cdrom_lockdoor
                   jsr cdrom_lockdoor
                   rts
ioctl_CDROMEJECT   = *
                   lda #2
                   sta flag
                   jsr cdrom_eject
                   jsr cdrom_eject
                   jsr cdrom_eject
                   rts
ioctl_CDROMCLOSETRAY = *
                   lda #3
                   sta flag
                   jsr cdrom_eject
                   jsr cdrom_eject
                   jsr cdrom_eject
                   rts
ioctl_CDROMLOCK    = *
                   lda #1
                   sta flag
                   jsr cdrom_lockdoor
                   jsr cdrom_lockdoor
                   jsr cdrom_lockdoor
                   rts

ioctl_CDROMSTART   = *
                   lda #1
                   sta flag
                   jsr cdrom_startstop
                   jsr cdrom_startstop
                   jsr cdrom_startstop
                   rts
                   
ioctl_CDROMSTOP    = *
                   lda #0
                   sta flag
                   jsr cdrom_startstop
                   jsr cdrom_startstop
                   jsr cdrom_startstop
                   rts
ioctl_CDROMPAUSE   = *
                   lda #0
                   sta flag
                   jsr cdrom_pause
                   jsr cdrom_pause
                   jsr cdrom_pause
                   rts
ioctl_CDROMRESUME  = *
                   lda #1
                   sta flag
                   jsr cdrom_pause
                   jsr cdrom_pause
                   jsr cdrom_pause
                   rts
;--------------------------------------
; ioctl_CDROMREADTOC
;
; Input: A - Hi byte of the TOC data buffer
;
;------------------------
; TOC data buffer format:
;
; byte
;  0  MSB TOC Data Length
;  1  LSB TOC Data Length
;  2  First Track Number
;  3  Last Track Number
;  -  TOC Track Descriptors
;  0  Reserved
;  1  (7-4) ADR; (3-0) Control
;  2  Track Number
;  3  Reserved
;  4  Reserved
;  5  M Field (Minute)
;  6  S Field (Second)
;  7  F Field (Frame)
;  -
; Control 00-03 2 Audio
;         04,06 Digital Data
;--------------------------------------

ioctl_CDROMREADTOC = *
                   sta out_buffer
                   jsr cdrom_read_toc
                   jsr cdrom_read_toc
                   jsr cdrom_read_toc
                   rts
;--------------------------------------
; ioctl_CDROMSUBCHNL
;
; Input: A - Hi byte of the Position data buffer
;
;-----------------------------
; Position data buffer format:
;
; byte
;  0  Reserved
;  1  Audio Status
;  2  MSB Sub-channel Data Length
;  3  LSB Sub-channel Data Length
;
;  4  Sub Channel Data Format Code -> $01
;  5  (7-4) ADR; (3-0) Control
;  6  Track Number
;  7  Index Number (index in the current track)
;  8  Reserved
;  9  M Field (Minute) Absolute
; 10  S Field (Second) Absolute
; 11  F Field (Frame) Absolute
; 12  Reserved
; 13  M Field (Minute) relative to the current track
; 14  S Field (Second)
; 15  F Field (Frame)
; (16-47 Identification data)
; 
;  Audio Status:
;  $00  Audio status byte not supported or not valid
;  $11  Play operation in progress
;  $12  Play operation paused
;  $13  Play operation successfully completed
;  $14  Play operation stopped due to error
;  $15  No current audio status to return
;--------------------------------------

ioctl_CDROMSUBCHNL = *
                   sta out_buffer
                   lda #1
                   sta flag
                   jsr cdrom_read_subchannel
                   jsr cdrom_read_subchannel
                   jsr cdrom_read_subchannel
                   rts
;--------------------------------------
; ioctl_CDROMPLAYMSF
;
; Input: X - Hi byte of the Start/End buffer
;
;--------------------------
;  Start/End buffer format:
;
; byte
;  0  Reserved
;  1  M Field (Minute) Start
;  2  S Field (Second) Start
;  3  F Filed (Frame) Start
;  0  Reserved
;  1  M Field (Minute) End
;  2  S Field (Second) End
;  3  F Filed (Frame) End
;--------------------------------------

ioctl_CDROMPLAYMSF = *
                   txa
                   jsr set_start_end_track

                   jsr cdrom_play_audio
                   jsr cdrom_play_audio
                   jsr cdrom_play_audio
                   rts
;--------------------------------------
; ioctl_CDROMREADDATA
;
; Input: A - Hi byte of the output buffer
;        X - Hi byte of the Sector Info buffer
;
;--------------------------
;  Start/End buffer format:
;
; byte
;  0  Reserved
;  1  M Field (Minute)
;  2  S Field (Second)
;  3  F Filed (Frame)
;--------------------------------------

ioctl_CDROMREADDATA = *
                   sta out_buffer
                   txa
                   jsr set_sector_to_read
                   
                   jsr cdrom_read_sector
                   jsr cdrom_read_sector
                   jsr cdrom_read_sector
                   rts

ioctl_CDROMPLAYTRKIND = *                  ;Plays all CD
                   lda #0                  ;clear Buffer
                   sta tocbuf_FirstTrk
                   sta tocbuf_LastTrk
                   lda #>toc_buffer        ;Buffer
                   sta out_buffer
                   jsr cdrom_read_toc
                   jsr cdrom_read_toc
                   jsr cdrom_read_toc
                   jsr set_start_end_all

                   jsr cdrom_play_audio
                   jsr cdrom_play_audio
                   jsr cdrom_play_audio
                   rts
;--------------------------------------
; ioctl_CDROMREADCAP
;
; Input: A - Hi byte of the output buffer
;
;-----------------------
;  output buffer format:
;
; byte
;  0  MSB Logical Block Address
;  1      Logical Block Address
;  2      Logical Block Address
;  3  LSB Logical Block Address
;  4  MSB Block Length in Bytes (2048d)
;  5      Block Length in Bytes
;  6      Block Length in Bytes
;  7  LSB Block Length in Bytes
;--------------------------------------

ioctl_CDROMREADCAP = *
                   sta out_buffer
                   jsr cdrom_read_capacity
                   jsr cdrom_read_capacity
                   jsr cdrom_read_capacity
                   rts

ioctl_CDROMRESET   = *
                   jmp atapi_reset

;--------------------------------------
; atapi_identify_drive
;
; Input: A - Hi byte of the buffer
;--------------------------------------

ioctl_IDENTIFYDRIVE = *
                   jsr atapi_reset         ;Reset ATAPI drive
                   jsr read_atapi_signature
                   lda #>identify_buffer
                   jsr atapi_identify_drive 
                   bcs exit_err
                   lda #>identify_buffer
                   jsr print_atapi_info
                   rts

ioctl_ANALYZECDROM = *
                   jsr check_sense
                   bcc exit_ok

                   lda #>atapi_request_sense  ;Buffer
                   sta out_buffer
                   jsr cdrom_request_sense
                   bcs exit_err
                   jsr cdrom_analyze_sense_data
                   rts

exit_ok            = *
                   port $00
                   rts
exit_err           = *
                   rts
;--------------------------------------
;--------------------------------------
;                lda #1                  ;1=Slave
;                sta drive
;                jsr atapiTOC
;                jsr print_atapi_TOC     ;cte z toc_bufferu
;--------------------------------------
check_sense    = *
               lda IDE_STATUS_REG
               lda low_reg
               lsr
               bcc check_sense_err_notd

               lda #<sense_key_info
               ldy #>sense_key_info
               jsr $ab1e

               lda IDE_ERROR_REG
               lda low_reg
               lsr
               lsr
               lsr
               lsr
               
               pha
               jsr print_hex

               lda #" "
               jsr chrout
               pla

               cmp #$10
               bcc check_sense_cont
               
               lda #$10          ;"BAD SENSE KEY"
               
check_sense_cont = *
               asl
               tax
               lda sense_key_texts+1,x
               tay 
               lda sense_key_texts,x
               jsr $ab1e
               sec
               rts
               
check_sense_err_notd = *
               lda #<check_no_info
               ldy #>check_no_info
               jsr $ab1e
               clc
               rts
               ;
check_no_info .text $0d,"NO ERROR",$00
;--------------------------------------
;### Print ATAPI info ###
;
; Input: A - Hi byte of buffer
;--------------------------------------
print_atapi_info = *
               sta pom2
               lda #$00
               sta pom1
               
               lda #<serialnum_info
               ldy #>serialnum_info
               jsr $ab1e

               ldy #20                ;Offset
               ldx #20
               jsr print_string
               
               lda #<firmware_info
               ldy #>firmware_info
               jsr $ab1e

               ldy #46                ;Offset
               ldx #8
               jsr print_string
               
               lda #<model_info
               ldy #>model_info
               jsr $ab1e

               ldy #54                ;Offset
               ldx #40
               jsr print_string
               rts
;------------------
print_string   = *
               lda (pom1),y
               jsr chrout
               iny
               dex
               bne print_string
               rts
               ;
serialnum_info .text $0d,"SERIAL NUMBER: ",$00
firmware_info  .text $0d,"FIRMWARE REVISION: ",$00
model_info     .text $0d,"MODEL NUMBER: ",$00
;--------------------------------------
;### Print ATAPI TOC ###
;
; Vyuziva 'toc_buffer'
;--------------------------------------
print_atapi_TOC = *
               lda #<tracknum_info
               ldy #>tracknum_info
               jsr $ab1e
                
               sec                      ;NumTracks=LastTrk-FirstTrk+1
               lda tocbuf_LastTrk
               sbc tocbuf_FirstTrk
               sta NumTracks
               inc NumTracks
               
               lda NumTracks
               jsr print_dec

               lda #<starting_info
               ldy #>starting_info
               jsr $ab1e
                                        ;Zatin jen na 30 tracku!
               lda NumTracks
               sta pom3
               
               ldy #0
track_print_loop = *
               iny
               iny
               lda toc_data_buffer,y     ;Track number
               jsr print_dec
               lda #$20
               jsr chrout
               iny
               iny
               iny
               lda toc_data_buffer,y
               jsr print_dec    ;Min
               lda #":"
               jsr chrout  
               iny
               lda toc_data_buffer,y
               jsr print_dec    ;Sec.
               lda #":"
               jsr chrout  
               iny
               lda toc_data_buffer,y
               jsr print_dec    ;Frame
               lda #$0d
               jsr chrout
               iny
               dec pom3
               bne track_print_loop
               rts
               ;
tracknum_info  .text $0d,"NUMBER OF TRACKS: ",$00
starting_info  .text $0d,"STARTING TIMES:",$0d,$00
;--------------------------------------
; ATAPI Reset Drive
;--------------------------------------
atapi_reset    = *
               lda drive
               asl
               asl
               asl
               asl
               sta low_reg
               sta IDE_SELECT_REG

               lda #atapi_reset_d      ;Identify ATAPI device
               sta low_reg
               sta IDE_COMMAND_REG
                               
               jsr tst_BUSY
               rts
;--------------------------------------
; Test ATAPI signature ($eb14)
;--------------------------------------
read_atapi_signature = *
               lda IDE_LCYL_REG
               lda low_reg
               cmp #$14
               bne atapi_not_detected
               lda IDE_HCYL_REG
               lda low_reg
               cmp #$eb
               bne atapi_not_detected
               lda #<detected_info
               ldy #>detected_info
               jsr $ab1e
               clc
               rts
atapi_not_detected = *
               lda #<notdetected_info
               ldy #>notdetected_info
               jsr $ab1e
               sec
               rts
               ;
detected_info  .text $0d,"ATAPI DETECTED",$0d,$00
notdetected_info .text $0d,"ATAPI NOT DETECTED",$0d,$00
;--------------------------------------
;### Identify ATAPI drive ###
;
; Input: A-buffer, drive (0/1)
;--------------------------------------
atapi_identify_drive = *
               
               sta pom2
               ldy #$00
               sty pom1

;              jsr tstbusy
;              bcs ATAPI_error

               lda drive
               asl
               asl
               asl
               asl
               sta low_reg
               sta IDE_SELECT_REG

;               jsr tst_DRDY
;               bcs ATAPI_error

               jsr tst_BUSY
               bcs atapi_error
         
               lda #atapi_ident_d      ;Identify ATAPI device
               sta low_reg
               sta IDE_COMMAND_REG
               
               jsr tst_DRQ              ;tst_DRQ
               bcs ATAPI_error

               ldy #$00

atapi_identify_loop = *

               lda IDE_DATA_REG

               lda hig_reg             ;8-15 BIT
               sta (pom1),y
               iny
               lda low_reg             ;0-7 BIT
               sta (pom1),y
               iny

               bne atapi_identify_inc
               inc pom2

atapi_identify_inc =*
               
               jsr ntst_DRQ
               bcs ATAPI_ok            ;DRQ=0
                                       ;-----------------------------------
                                       ;Pokud HDD je pripojen, ale neni
                                       ;zapnut, tato smycka by neskoncila
                                       ;-----------------------------------
;               ldx #$04                ;0
;               inx                     ;1
;               inx                     ;2
;               cpx pom2                ;2
;               beq ATAPI_error         ;HDD neni zapnut!!
               jmp atapi_identify_loop
;------------------
ATAPI_ok       = *
               clc
               rts
ATAPI_error    = *
               sec
               rts
ATAPI_error_99 = *
               port $99
               rts
ATAPI_error_88 = *
               port $88
               rts
ATAPI_error_77 = *
               port $77
               rts
ATAPI_error_aa = *
               port $aa
               rts
;--------------------------------------
; Input: A - buffer (hi-byte)
;--------------------------------------
cdrom_read_buffer = *
               sta pom2
               ldy #$00
               sty pom1
atapi_read_loop = *   

               lda IDE_DATA_REG
               
               lda low_reg             ;0-7 BIT
               sta (pom1),y
               iny
               lda hig_reg             ;8-15 BIT
               sta (pom1),y
               iny

               bne atapi_read_inc
               inc pom2
atapi_read_inc =*
               
               jsr ntst_DRQ
               bcs ATAPI_ok
               bcc atapi_read_loop

; Delku bloku k nacteni obsahuji registry cyllow_g, cylhi_g
; ---------------------------------------------------------

;--------------------------------------
cdrom_analyze_sense_data = *
               
               lda #<error_code_info
               ldy #>error_code_info
               jsr $ab1e
               
               lda reqbuf_error_code
               jsr print_hex
               
               lda #<sense_key_info
               ldy #>sense_key_info
               jsr $ab1e
               
               lda reqbuf_sense_key
               jsr print_hex
               
               lda #" "
               jsr chrout
               
               lda reqbuf_sense_key
               cmp #$10
               bcc analyze_cont0
               
               lda #$10          ;"BAD SENSE KEY"
               
analyze_cont0  = *
               asl
               tax
               lda sense_key_texts+1,x
               tay 
               lda sense_key_texts,x
               jsr $ab1e

               lda reqbuf_asc
               cmp #$40
               bne analyze_cont

               lda #<diagnostic_failure_info
               ldy #>diagnostic_failure_info
               jsr $ab1e
               
               lda reqbuf_ascq
               jsr print_hex
               rts
analyze_cont   = *
               lda #<additional_sense_info
               ldy #>additional_sense_info
               jsr $ab1e
               
               lda reqbuf_asc
               jsr print_hex
               lda #" "
               jsr chrout
               lda #"$"
               jsr chrout
               
               lda reqbuf_ascq
               jsr print_hex
               rts
;--------------------------------------
error_code_info         .text $0d,"  ERROR CODE: $",$00
sense_key_info          .text $0d,"  SENSE KEY: $",$00
diagnostic_failure_info .text $0d,"DIAGNOSTIC FAILURE ON COMPONENT $",$00
additional_sense_info   .text $0d,"  ADDITIONAL SENSE DATA $",$00
;------------------
sense_key_texts         = *
                        .word sense_key_0
                        .word sense_key_1
                        .word sense_key_2
                        .word sense_key_3
                        .word sense_key_4
                        .word sense_key_5
                        .word sense_key_6
                        .word sense_key_7
                        .word sense_key_8
                        .word sense_key_9
                        .word sense_key_a
                        .word sense_key_b
                        .word sense_key_c
                        .word sense_key_d
                        .word sense_key_e
                        .word sense_key_f
                        .word bad_sense_key_info
sense_key_0             .text "NO SENSE DATA",$00
sense_key_1             .text "RECOVERED ERROR",$00
sense_key_2             .text "NOT READY",$00
sense_key_3             .text "MEDIUM ERROR",$00
sense_key_4             .text "HARDWARE ERROR",$00
sense_key_5             .text "ILLEGAL REQUEST",$00
sense_key_6             .text "UNIT ATTENTION",$00
sense_key_7             .text "DATA PROTECT",$00
sense_key_8             = *
sense_key_9             = *
sense_key_a             = *
sense_key_c             = *
sense_key_d             = *
sense_key_f             .text "(RESERVED)",$00
sense_key_b             .text "ABORTED COMMAND",$00
sense_key_e             .text "MISCOMPARE",$00
bad_sense_key_info      .text "(BAD SENSE KEY)",$00
;--------------------------------------
; Input A - byte
;--------------------------------------
print_hex      = *
               pha
               lsr
               lsr
               lsr
               lsr
               tay
               lda table_hex,y
               jsr chrout
               pla
               and #$0f
               tay
               lda table_hex,y
               jsr chrout
               rts
table_hex      .text "0123456789ABCDEF"
;---------------------------------------
; Input A - byte
;---------------------------------------
print_bin      = *
               ldy #8
               pha
print_bin_loop = *
               pla
               asl
               pha
               bcc print_zero
               lda #"1"
               .byte $2c
print_zero     = *
               lda #"0"
               jsr chrout
               dey
               bne print_bin_loop
               pla
               rts
;---------------------------------------
; Input A - byte           (zachovava Y)
;---------------------------------------
print_dec      = *
               tax
               tya
               pha
               lda #0
               jsr $bdcd
               pla
               tay
               rts
;;--------------------------------------
;;### ATAPI Read block ###
;;
;; Input: A-buffer, drive (0/1)
;;--------------------------------------
;atapi_read_block = *
;               sta pom3
;;------------------                     Set Command Buffer
;               ldy #0
;               ;
;               lda #$be                ;Read CD
;               sta comm_buff,y
;               iny
;               lda #$00
;               sta comm_buff,y
;               ;
;               ldy #2
;               lda #$00
;               sta comm_buff,y            ;LBA (MSB)
;               iny
;               lda #$00
;               sta comm_buff,y
;               iny
;               lda #$00
;               sta comm_buff,y
;               iny
;               lda #$00
;               sta comm_buff,y
;               ;
;               ldy #6
;               lda #$00
;               sta comm_buff,y            ;Transfer Length (MSB)
;               iny
;               lda #$00
;               sta comm_buff,y
;               iny
;               lda #$01
;               sta comm_buff,y
;               iny
;               lda #$00                ;Flag Bits
;               sta comm_buff,y
;               iny
;               lda #$00                ;Syb channel bits
;               sta comm_buff,y
;;------------------
;                
;               jsr cdrom_start_packet_command
;               bcs atapi_error1
;               jsr cdrom_transfer_packet_command
;               bcs atapi_error1
;;               jsr tst_ALTBUSY
;               jsr tst_DRQ
;               bcs ATAPI_error1_ppp        ;DRQ=0 -> Command terminated
;               
;               lda #$00
;               sta pom1
;               lda pom3
;               sta pom2
;
;               jmp atapi_read_buffer

;--------------------------------------
; Input: flag
;                      0 - pause
;                      1 - resume
;--------------------------------------
cdrom_pause    = *
               memset 0
               pc 0,ATAPI_CMD_PAUSE
               pm 8,flag
                              
               jsr cdrom_packet_command
               rts
;--------------------------------------
;Input: flag
;                      2 - eject the disk
;                      3 - load the disk
;--------------------------------------
cdrom_eject    = *
               memset 0
               pc 0,ATAPI_CMD_START_STOP
               pm 4,flag
               
               jsr cdrom_packet_command
               rts
;--------------------------------------
;Input: flag
;                      0 - stop the disk
;                      1 - start the disk, read TOC
;--------------------------------------
cdrom_startstop = *
               memset 0
               pc 0,ATAPI_CMD_START_STOP
               pc 1,1
               pm 4,flag
               
               jsr cdrom_packet_command
               rts
;--------------------------------------
;Input: flag
;                      0 - unlock door
;--------------------------------------
cdrom_lockdoor = *
               memset 0
               pc 0,ATAPI_CMD_LOCK_UNLOCK
               pm 4,flag
               
               jsr cdrom_packet_command
               rts
;--------------------------------------
;Input: drive, out_buffer
;--------------------------------------
cdrom_read_capacity = *

               memset 0
               pc 0,ATAPI_READ_CAPACITY

               jsr cdrom_packet_command
               bcs cdrom_read_capacity_error
               lda out_buffer
               jsr cdrom_read_buffer
cdrom_read_capacity_error = *
               rts
;--------------------------------------
;Input: out_buffer
;--------------------------------------
cdrom_read_toc = *
               memset 0
               pc 0,ATAPI_CMD_READTOC
               pc 1,2                       ;use MSF
;               pc 2,0                       ;Format
;               pc 6,0                       ;Starting track

                        ;Vychazim z toho, ze vice jak 30 tracku nebude               

               pc 7,$00       ;30x8+4       ;Hi buflen
               pc 8,$ff                     ;Lo buflen
;               pc 9,0                       ;Format
                                            ;When Format in Byte 2 is 0
                                            ;then Byte 9 is used
               jsr cdrom_packet_command
               bcs atapiTOC_error
               lda out_buffer
               jsr cdrom_read_buffer
atapiTOC_error = *
               rts
;--------------------------------------
;Input: out_buffer, flag
;
;                      1 - Audio Status
;
;--------------------------------------
cdrom_read_subchannel = *
                      
               memset 0
               pc 0,ATAPI_READ_SUB
               pc 1,2                       ;use MSF
               pc 2,$40                     ;read sub Q
               pm 3,flag
               pc 7,$00                     ;Hi buffer
               pc 8,$ff                     ;Lo buffer

               jsr cdrom_packet_command
               bcs cdrom_read_subchannel_error
               lda out_buffer
               jsr cdrom_read_buffer
cdrom_read_subchannel_error = *
               rts
;--------------------------------------
cdrom_read_sector = *
               memset 0
               pc 0,ATAPI_CMD_READ10

               pm 3,Start_M
               pm 4,Start_S
               pm 5,Start_F
                      
               pc 7,$00     ;2048>>8
               pc 8,$01     ;2048

               jsr cdrom_packet_command
               bcs cdrom_read_sector_error
               lda out_buffer
               jsr cdrom_read_buffer
cdrom_read_sector_error = *
               rts

;--------------------------------------
set_sector_to_read = *
               sta pom2
               lda #$00
               sta pom1
               ldy #1
               lda (pom1),y
               sta Start_M
               iny
               lda (pom1),y
               sta Start_S
               iny
               lda (pom1),y
               sta Start_F
               rts
;--------------------------------------
set_start_end_track = *
               sta pom2
               lda #$00
               sta pom1
               ldy #1
               lda (pom1),y
               sta Start_M
               iny
               lda (pom1),y
               sta Start_S
               iny
               lda (pom1),y
               sta Start_F
               iny
               iny
               lda (pom1),y
               sta End_M
               iny
               lda (pom1),y
               sta End_S
               iny
               lda (pom1),y
               sta End_F
               rts
;--------------------------------------
; vyuziva 'toc_data_buffer'
;--------------------------------------
set_start_end_all = *
               ldy tocbuf_FirstTrk
               dey                          ;TrackNum-1
               tya
               asl
               asl
               asl ;x8
               clc
               adc #5  ;Posun na Min
               tay
               lda toc_data_buffer,y
               sta Start_M
               iny
               lda toc_data_buffer,y
               sta Start_S
               iny
               lda toc_data_buffer,y
               sta Start_F

               ldy tocbuf_LastTrk
               tya
               asl
               asl
               asl ;x8
               clc
               adc #5  ;Posun na End_Min
               tay
               lda toc_data_buffer,y
               sta End_M
               iny
               lda toc_data_buffer,y
               sta End_S
               iny
               lda toc_data_buffer,y
               sta End_F
               rts
;--------------------------------------
cdrom_play_audio = *
               memset 0
               pc 0,ATAPI_CMD_PLAY

               pm 3,Start_M
               pm 4,Start_S
               pm 5,Start_F
               pm 6,End_M
               pm 7,End_S
               pm 8,End_F

;               pc 3,0
;               pc 4,0
;               pc 5,0
;               pc 6,$15
;               pc 7,0
;               pc 8,0
               
               jsr cdrom_packet_command
               rts

;--------------------------------------
cdrom_check_status = *
               memset 0
               pc 0,TEST_UNIT_READY
               
               jsr cdrom_packet_command
               rts
;--------------------------------------
;Tato funkce neodrazi presne puvodni...
;--------------------------------------
cdrom_packet_command = *

               jsr cdrom_start_packet_command
               bcs atapi_error_11
               jsr cdrom_transfer_packet_command
               bcs atapi_error_22
               
               jsr tst_BUSY
               bcs atapi_error_33

               lda IDE_STATUS_REG
               lda low_reg
               and #$08
               sta pom1
               lda IDE_NSECTOR_REG      ;Interrupt reason register
               lda low_reg
               and #$03
               ora pom1
               sta service_port
               
               cmp #ATAPI_PH_DONE       ;3
               bne cdrom_pac_cont1
               clc
               return 0

cdrom_pac_cont1 = *
               cmp #ATAPI_PH_ABORT
               beq ATAPI_error_44

               clc                      ;Predpokladame data-in
               return 0

ATAPI_error_11 = *
               port $11
               return 1
ATAPI_error_22 = *
               port $22
               inc $d020
               return 1
ATAPI_error_33 = *
               port $33
               return 1
ATAPI_error_44 = *
               port $44
               return 1
;--------------------------------------
;### ATAPI Request sense ###
;Input: drive, out_buffer
;--------------------------------------
cdrom_request_sense = *
               memset 0
               pc 0,REQUEST_SENSE
               pc 4,18                 ;Delka vystupu

               jsr cdrom_packet_command
               lda out_buffer
               jsr cdrom_read_buffer
               rts
;--------------------------------------
cdrom_inquiry  = *
               memset 0
               pc 0,INQUIRY
               pc 4,36                 ;Delka vystupu

               jsr cdrom_packet_command
               lda out_buffer
               jsr cdrom_read_buffer
               rts
;--------------------------------------
;tst_DRDY  JSR tst_BUSY
;          bcs NOREADY
;          LDA IDE_STATUS_REG        ;STATUS REG.
;          LDA LOW_REG
;          AND #%01000000
;          BEQ NOREADY
;          clc
;          RTS
;NOREADY   = *
;          SEC
;          RTS
;------------------
;TSTERR   JSR TSTBUSY
;        ; bcs TSTERR
;         LDA IDE_STATUS_REG ;STATUS REG.
;         LDA LOW_REG
;         AND #%00000001
;         BEQ ERRTSTNO
;
;         LDA IDE_STATUS_REG ;STATUS REG.
;         LDA LOW_REG
;         STA 1024+38
;
;         LDA ERROR_G  ;ERROR REG.
;         LDA LOW_REG
;         STA 1025+38
;
;         jmp ERRYES
;ERRTSTNO = *
;         RTS
;--------------------------------------
;erryes   = *
;         inc $d020
;         jsr wait
;         dec $d020
;         rts
;------------------
; wait     = *
; ;         tya          ;save Y
;          ldx #0
; wait_loop2 = *
;          ldy #0
; wait_loop1 = *
;          nop
;          nop
;          nop
;          dey
;          bne wait_loop1
;          dex
;          bne wait_loop2
; ;         tay          ;back Y
;          rts
;------------------------------------------------------------------------------
;### Test BUSY ### C=0 -> BUSY=0
;------------------------------------------------------------------------------
tst_ALTBUSY    = *
               sta scpu_normal
               lda #5            ;Timeout
               sta pom8
ide_altwait_0     = *
               ldx #0
ide_altwait_1     = *         
               ldy #0
ide_altwait_2     = *
               lda IDE_ALTSTATUS_REG
               lda low_reg
               asl
               bcc ide_wait_stat_0
;--------
               dey
               bne ide_altwait_2
               dex
               bne ide_altwait_1
               dec pom8
               bne ide_altwait_0
               sec
               sta scpu_turbo
               rts
;--------------------------------------
tst_BUSY       = *
ide_wait_stat_BUSY_STAT = *
               sta scpu_normal
               lda #10            ;Timeout
               sta pom8
ide_wait_0     = *
               ldx #0
ide_wait_1     = *         
               ldy #0
ide_wait_2     = *
               lda IDE_STATUS_REG
               lda low_reg
               asl
               bcc ide_wait_stat_0
;--------
               dey
               bne ide_wait_2
               dex
               bne ide_wait_1
               dec pom8
               bne ide_wait_0
ide_wait_stat_1 = *
               sec
               sta scpu_turbo
               rts
ide_wait_stat_0 = *
               sta scpu_turbo
               rts
;------------------------------------------------------------------------------
;### Test DRQ+BUSY ### C=0 -> BUSY=0 and DRQ=1
;                      C=1 -> BUSY=0 and DRQ=0   or   BUSY=1 (timeout)
;------------------------------------------------------------------------------
tst_DRQ        = *
ide_wait_stat_DRQ_STAT = *
               jsr ide_wait_stat_BUSY_STAT
               bcs ide_wait_stat_0
ntst_DRQ       = *
               lda IDE_STATUS_REG
               lda low_reg
               and #%00001000
               beq ide_wait_stat_1
               clc
               rts
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
cdrom_start_packet_command = *

                           ;Wait for the controller to be idle.

               jsr ide_wait_stat_BUSY_STAT
               bcs cdrom_exit1

;---                       Select Drive
               lda drive
               asl
               asl
               asl
               asl
;               ora #%01000000     ; LBA mode
               sta low_reg
               sta IDE_SELECT_REG
;---
                           ;Set up the controller registers.

               lda #$00
               sta low_reg
               sta IDE_FEATURE_REG
               lda #$00
               sta low_reg
               sta IDE_NSECTOR_REG
               lda #$00
               sta low_reg
               sta IDE_SECTOR_REG
                               
                           ;Maximum packet size - mel jsem tu ($2000)

               lda #$00
               sta low_reg
               sta IDE_LCYL_REG
               lda #$80
               sta low_reg
               sta IDE_HCYL_REG

               lda #%00001010
               sta low_reg
               sta IDE_CONTROL_REG

                           ;Packet Command

               lda #WIN_PACKETCMD
               sta low_reg
               sta IDE_COMMAND_REG
               clc
cdrom_exit1    = *
               rts
;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
cdrom_transfer_packet_command = *

               jsr tst_ALTBUSY
               bcs cdrom_transfer_exit1
               jsr ide_wait_stat_DRQ_STAT
               bcs cdrom_transfer_exit1

                              ;Send the command to the device.

               jsr cdrom_out_packet
               clc
cdrom_transfer_exit1 = *
               rts
;------------------------------------------------------------------------------
;Input: drive, A=buffer
;------------------------------------------------------------------------------
cdrom_out_packet = *
               ldy #0
cdrom_out_loop = *   
               lda comm_buff,y
               sta low_reg
               lda comm_buff+1,y
               sta hig_reg
               sta IDE_DATA_REG     ;Je 16-ti bitovy

               iny
               iny
               cpy #12              ;Packet size 12 bajtu (6 words)
               bne cdrom_out_loop
               rts
;------------------------------------------------------------------------------

